@@ -21,6 +21,7 @@
 #include "fu-hwids-private.h"
 #include "fu-path.h"
 #include "fu-pefile-firmware.h"
+#include "fu-volume-locker.h"
 #include "fu-volume-private.h"
 
 /**
@@ -79,17 +80,14 @@ fu_context_get_fdt_file(GError **error)
 {
 	g_autofree gchar *fdtfn_local = NULL;
 	g_autofree gchar *fdtfn_sys = NULL;
-	g_autofree gchar *localstatedir_pkg = fu_path_from_kind(FU_PATH_KIND_LOCALSTATEDIR_PKG);
-	g_autofree gchar *sysfsdir = NULL;
 
 	/* look for override first, fall back to system value */
-	fdtfn_local = g_build_filename(localstatedir_pkg, "system.dtb", NULL);
+	fdtfn_local = fu_path_build(FU_PATH_KIND_LOCALSTATEDIR_PKG, "system.dtb", NULL);
 	if (g_file_test(fdtfn_local, G_FILE_TEST_EXISTS))
 		return g_file_new_for_path(fdtfn_local);
 
 	/* actual hardware value */
-	sysfsdir = fu_path_from_kind(FU_PATH_KIND_SYSFSDIR_FW);
-	fdtfn_sys = g_build_filename(sysfsdir, "fdt", NULL);
+	fdtfn_sys = fu_path_build(FU_PATH_KIND_SYSFSDIR_FW, "fdt", NULL);
 	if (g_file_test(fdtfn_sys, G_FILE_TEST_EXISTS))
 		return g_file_new_for_path(fdtfn_sys);
 
@@ -1109,6 +1107,65 @@ fu_context_hwid_quirk_cb(FuContext *self,
 	}
 }
 
+static void
+fu_context_detect_container(FuContext *self)
+{
+	gsize bufsz = 0;
+	g_autofree gchar *buf = NULL;
+
+	if (!g_file_get_contents("/proc/1/cgroup", &buf, &bufsz, NULL))
+		return;
+	if (g_strstr_len(buf, (gssize)bufsz, "docker") != NULL) {
+		fu_context_add_flag(self, FU_CONTEXT_FLAG_IS_CONTAINER);
+		return;
+	}
+	if (g_strstr_len(buf, (gssize)bufsz, "lxc") != NULL) {
+		fu_context_add_flag(self, FU_CONTEXT_FLAG_IS_CONTAINER);
+		return;
+	}
+}
+
+static void
+fu_context_detect_hypervisor_privileged(FuContext *self)
+{
+	g_autofree gchar *xen_privileged_fn = NULL;
+
+	/* privileged xen can access most hardware */
+	xen_privileged_fn = fu_path_build(FU_PATH_KIND_SYSFSDIR_FW_ATTRIB,
+					  "hypervisor",
+					  "start_flags",
+					  "privileged",
+					  NULL);
+	if (!g_file_test(xen_privileged_fn, G_FILE_TEST_EXISTS)) {
+		g_autofree gchar *contents = NULL;
+		if (g_file_get_contents(xen_privileged_fn, &contents, NULL, NULL)) {
+			if (g_strcmp0(contents, "1") == 0) {
+				fu_context_add_flag(self, FU_CONTEXT_FLAG_IS_HYPERVISOR_PRIVILEGED);
+				return;
+			}
+		}
+	}
+}
+
+static void
+fu_context_detect_hypervisor(FuContext *self)
+{
+	const gchar *flags;
+	g_autoptr(GHashTable) cpu_attrs = NULL;
+
+	cpu_attrs = fu_cpu_get_attrs(NULL);
+	if (cpu_attrs == NULL)
+		return;
+	flags = g_hash_table_lookup(cpu_attrs, "flags");
+	if (flags == NULL)
+		return;
+	if (g_strstr_len(flags, -1, "hypervisor") != NULL) {
+		fu_context_add_flag(self, FU_CONTEXT_FLAG_IS_HYPERVISOR);
+		fu_context_detect_hypervisor_privileged(self);
+		return;
+	}
+}
+
 /**
  * fu_context_load_hwinfo:
  * @self: a #FuContext
@@ -1131,6 +1188,7 @@ fu_context_load_hwinfo(FuContext *self,
 	FuContextPrivate *priv = GET_PRIVATE(self);
 	FuConfigLoadFlags config_load_flags = FU_CONFIG_LOAD_FLAG_NONE;
 	GPtrArray *guids;
+	const gchar *machine_kind = g_getenv("FWUPD_MACHINE_KIND");
 	g_autoptr(GError) error_hwids = NULL;
 	g_autoptr(GError) error_bios_settings = NULL;
 	struct {
@@ -1154,7 +1212,8 @@ fu_context_load_hwinfo(FuContext *self,
 	fu_progress_add_step(progress, FWUPD_STATUS_LOADING, 1, "hwids-setup");
 	fu_progress_add_step(progress, FWUPD_STATUS_LOADING, 3, "set-flags");
 	fu_progress_add_step(progress, FWUPD_STATUS_LOADING, 1, "detect-fde");
-	fu_progress_add_step(progress, FWUPD_STATUS_LOADING, 94, "reload-bios-settings");
+	fu_progress_add_step(progress, FWUPD_STATUS_LOADING, 1, "detect-hypervisor-container");
+	fu_progress_add_step(progress, FWUPD_STATUS_LOADING, 93, "reload-bios-settings");
 
 	/* required always */
 	if (flags & FU_CONTEXT_HWID_FLAG_WATCH_FILES)
@@ -1198,6 +1257,28 @@ fu_context_load_hwinfo(FuContext *self,
 	fu_context_detect_full_disk_encryption(self);
 	fu_progress_step_done(progress);
 
+	/* allow overriding for development */
+	if (machine_kind != NULL) {
+		if (g_strcmp0(machine_kind, "physical") == 0) {
+			/* nothing to do */
+		} else if (g_strcmp0(machine_kind, "container") == 0) {
+			fu_context_add_flag(self, FU_CONTEXT_FLAG_IS_CONTAINER);
+		} else if (g_strcmp0(machine_kind, "virtual") == 0) {
+			fu_context_add_flag(self, FU_CONTEXT_FLAG_IS_HYPERVISOR);
+		} else {
+			g_set_error(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_DATA,
+				    "invalid machine kind specified: %s",
+				    machine_kind);
+			return FALSE;
+		}
+	} else {
+		fu_context_detect_hypervisor(self);
+		fu_context_detect_container(self);
+	}
+	fu_progress_step_done(progress);
+
 	fu_context_add_udev_subsystem(self, "firmware-attributes", NULL);
 	if (!fu_context_reload_bios_settings(self, &error_bios_settings))
 		g_debug("%s", error_bios_settings->message);
@@ -1779,11 +1860,11 @@ fu_context_get_default_esp(FuContext *self, GError **error)
 			FuVolume *esp = g_ptr_array_index(esp_volumes, i);
 			guint score = 0;
 			g_autofree gchar *kind = NULL;
-			g_autoptr(FuDeviceLocker) locker = NULL;
+			g_autoptr(FuVolumeLocker) locker = NULL;
 			g_autoptr(GError) error_local = NULL;
 
 			/* ignore the volume completely if we cannot mount it */
-			locker = fu_volume_locker(esp, &error_local);
+			locker = fu_volume_locker_new(esp, &error_local);
 			if (locker == NULL) {
 				g_warning("failed to mount ESP: %s", error_local->message);
 				continue;
@@ -1841,10 +1922,10 @@ fu_context_get_default_esp(FuContext *self, GError **error)
 
 	if (esp_volumes->len == 1) {
 		FuVolume *esp = g_ptr_array_index(esp_volumes, 0);
-		g_autoptr(FuDeviceLocker) locker = NULL;
+		g_autoptr(FuVolumeLocker) locker = NULL;
 
 		/* ensure it can be mounted */
-		locker = fu_volume_locker(esp, error);
+		locker = fu_volume_locker_new(esp, error);
 		if (locker == NULL)
 			return NULL;
 
@@ -1962,7 +2043,7 @@ fu_context_get_esp_files_for_entry(FuContext *self,
 	g_autofree gchar *filename = NULL;
 	g_autofree gchar *mount_point = NULL;
 	g_autofree gchar *shim_name = fu_context_build_uefi_basename_for_arch("shim");
-	g_autoptr(FuDeviceLocker) volume_locker = NULL;
+	g_autoptr(FuVolumeLocker) volume_locker = NULL;
 	g_autoptr(FuEfiFilePathDevicePath) dp_path = NULL;
 	g_autoptr(FuEfiHardDriveDevicePath) dp_hdd = NULL;
 	g_autoptr(FuFirmware) dp_list = NULL;
@@ -2001,7 +2082,7 @@ fu_context_get_esp_files_for_entry(FuContext *self,
 				    "cannot mount volume by policy");
 		return FALSE;
 	}
-	volume_locker = fu_volume_locker(volume, error);
+	volume_locker = fu_volume_locker_new(volume, error);
 	if (volume_locker == NULL)
 		return FALSE;
 	dp_filename = fu_efi_file_path_device_path_get_name(dp_path, error);
