@@ -48,6 +48,7 @@ typedef struct {
 	guint depth;
 	GPtrArray *chunks;  /* nullable, element-type FuChunk */
 	GPtrArray *patches; /* nullable, element-type FuFirmwarePatch */
+	GPtrArray *magic;   /* nullable, element-type FuFirmwarePatch */
 } FuFirmwarePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE(FuFirmware, fu_firmware, G_TYPE_OBJECT)
@@ -57,76 +58,6 @@ enum { PROP_0, PROP_PARENT, PROP_LAST };
 
 #define FU_FIRMWARE_IMAGE_DEPTH_MAX 50
 
-/**
- * fu_firmware_flag_to_string:
- * @flag: a #FuFirmwareFlags, e.g. %FU_FIRMWARE_FLAG_DEDUPE_ID
- *
- * Converts a #FuFirmwareFlags to a string.
- *
- * Returns: identifier string
- *
- * Since: 1.5.0
- **/
-const gchar *
-fu_firmware_flag_to_string(FuFirmwareFlags flag)
-{
-	if (flag == FU_FIRMWARE_FLAG_NONE)
-		return "none";
-	if (flag == FU_FIRMWARE_FLAG_DEDUPE_ID)
-		return "dedupe-id";
-	if (flag == FU_FIRMWARE_FLAG_DEDUPE_IDX)
-		return "dedupe-idx";
-	if (flag == FU_FIRMWARE_FLAG_HAS_CHECKSUM)
-		return "has-checksum";
-	if (flag == FU_FIRMWARE_FLAG_HAS_VID_PID)
-		return "has-vid-pid";
-	if (flag == FU_FIRMWARE_FLAG_DONE_PARSE)
-		return "done-parse";
-	if (flag == FU_FIRMWARE_FLAG_HAS_STORED_SIZE)
-		return "has-stored-size";
-	if (flag == FU_FIRMWARE_FLAG_ALWAYS_SEARCH)
-		return "always-search";
-	if (flag == FU_FIRMWARE_FLAG_NO_AUTO_DETECTION)
-		return "no-auto-detection";
-	if (flag == FU_FIRMWARE_FLAG_HAS_CHECK_COMPATIBLE)
-		return "has-check-compatible";
-	return NULL;
-}
-
-/**
- * fu_firmware_flag_from_string:
- * @flag: a string, e.g. `dedupe-id`
- *
- * Converts a string to a #FuFirmwareFlags.
- *
- * Returns: enumerated value
- *
- * Since: 1.5.0
- **/
-FuFirmwareFlags
-fu_firmware_flag_from_string(const gchar *flag)
-{
-	if (g_strcmp0(flag, "dedupe-id") == 0)
-		return FU_FIRMWARE_FLAG_DEDUPE_ID;
-	if (g_strcmp0(flag, "dedupe-idx") == 0)
-		return FU_FIRMWARE_FLAG_DEDUPE_IDX;
-	if (g_strcmp0(flag, "has-checksum") == 0)
-		return FU_FIRMWARE_FLAG_HAS_CHECKSUM;
-	if (g_strcmp0(flag, "has-vid-pid") == 0)
-		return FU_FIRMWARE_FLAG_HAS_VID_PID;
-	if (g_strcmp0(flag, "done-parse") == 0)
-		return FU_FIRMWARE_FLAG_DONE_PARSE;
-	if (g_strcmp0(flag, "has-stored-size") == 0)
-		return FU_FIRMWARE_FLAG_HAS_STORED_SIZE;
-	if (g_strcmp0(flag, "always-search") == 0)
-		return FU_FIRMWARE_FLAG_ALWAYS_SEARCH;
-	if (g_strcmp0(flag, "no-auto-detection") == 0)
-		return FU_FIRMWARE_FLAG_NO_AUTO_DETECTION;
-	if (g_strcmp0(flag, "has-check-compatible") == 0)
-		return FU_FIRMWARE_FLAG_HAS_CHECK_COMPATIBLE;
-	return FU_FIRMWARE_FLAG_NONE;
-}
-
 typedef struct {
 	gsize offset;
 	GBytes *blob;
@@ -141,7 +72,7 @@ fu_firmware_patch_free(FuFirmwarePatch *ptch)
 
 /**
  * fu_firmware_add_flag:
- * @firmware: a #FuFirmware
+ * @self: a #FuFirmware
  * @flag: the firmware flag
  *
  * Adds a specific firmware flag to the firmware.
@@ -149,16 +80,16 @@ fu_firmware_patch_free(FuFirmwarePatch *ptch)
  * Since: 1.5.0
  **/
 void
-fu_firmware_add_flag(FuFirmware *firmware, FuFirmwareFlags flag)
+fu_firmware_add_flag(FuFirmware *self, FuFirmwareFlags flag)
 {
-	FuFirmwarePrivate *priv = GET_PRIVATE(firmware);
-	g_return_if_fail(FU_IS_FIRMWARE(firmware));
+	FuFirmwarePrivate *priv = GET_PRIVATE(self);
+	g_return_if_fail(FU_IS_FIRMWARE(self));
 	priv->flags |= flag;
 }
 
 /**
  * fu_firmware_has_flag:
- * @firmware: a #FuFirmware
+ * @self: a #FuFirmware
  * @flag: the firmware flag
  *
  * Finds if the firmware has a specific firmware flag.
@@ -168,10 +99,10 @@ fu_firmware_add_flag(FuFirmware *firmware, FuFirmwareFlags flag)
  * Since: 1.5.0
  **/
 gboolean
-fu_firmware_has_flag(FuFirmware *firmware, FuFirmwareFlags flag)
+fu_firmware_has_flag(FuFirmware *self, FuFirmwareFlags flag)
 {
-	FuFirmwarePrivate *priv = GET_PRIVATE(firmware);
-	g_return_val_if_fail(FU_IS_FIRMWARE(firmware), FALSE);
+	FuFirmwarePrivate *priv = GET_PRIVATE(self);
+	g_return_val_if_fail(FU_IS_FIRMWARE(self), FALSE);
 	return (priv->flags & flag) > 0;
 }
 
@@ -261,7 +192,7 @@ fu_firmware_set_version_raw(FuFirmware *self, guint64 version_raw)
 	if (klass->convert_version != NULL) {
 		g_autofree gchar *version = klass->convert_version(self, version_raw);
 		if (version != NULL)
-			fu_firmware_set_version(self, version);
+			fu_firmware_set_version(self, version); /* nocheck:set-version */
 	}
 }
 
@@ -308,7 +239,7 @@ fu_firmware_set_version_format(FuFirmware *self, FwupdVersionFormat version_form
 	/* convert this, now we know */
 	if (klass->convert_version != NULL && priv->version != NULL && priv->version_raw != 0) {
 		g_autofree gchar *version = klass->convert_version(self, priv->version_raw);
-		fu_firmware_set_version(self, version);
+		fu_firmware_set_version(self, version); /* nocheck:set-version */
 	}
 }
 
@@ -887,6 +818,35 @@ fu_firmware_add_chunk(FuFirmware *self, FuChunk *chk)
 	g_ptr_array_add(priv->chunks, g_object_ref(chk));
 }
 
+/**
+ * fu_firmware_add_magic:
+ * @self: a #FuFirmware
+ * @buf: some data
+ * @bufsz: sizeof @buf
+ * @offset: offset to start parsing, typically 0x0
+ *
+ * Adds a possible magic signature to the image.
+ *
+ * Since: 2.0.18
+ **/
+void
+fu_firmware_add_magic(FuFirmware *self, const guint8 *buf, gsize bufsz, gsize offset)
+{
+	FuFirmwarePrivate *priv = GET_PRIVATE(self);
+	g_autofree FuFirmwarePatch *patch = g_new0(FuFirmwarePatch, 1);
+
+	g_return_if_fail(FU_IS_FIRMWARE(self));
+	g_return_if_fail(buf != NULL);
+	g_return_if_fail(bufsz != 0);
+
+	if (priv->magic == NULL)
+		priv->magic =
+		    g_ptr_array_new_with_free_func((GDestroyNotify)fu_firmware_patch_free);
+	patch->blob = g_bytes_new(buf, bufsz);
+	patch->offset = offset;
+	g_ptr_array_add(priv->magic, g_steal_pointer(&patch));
+}
+
 /**
  * fu_firmware_get_checksum:
  * @self: a #FuPlugin
@@ -921,17 +881,26 @@ fu_firmware_get_checksum(FuFirmware *self, GChecksumType csum_kind, GError **err
 		}
 	}
 
+	/* write */
+	if (klass->write != NULL) {
+		blob = fu_firmware_write(self, error);
+		if (blob == NULL)
+			return NULL;
+		return g_compute_checksum_for_bytes(csum_kind, blob);
+	}
+
 	/* internal data */
 	if (priv->bytes != NULL)
 		return g_compute_checksum_for_bytes(csum_kind, priv->bytes);
 	if (priv->stream != NULL)
 		return fu_input_stream_compute_checksum(priv->stream, csum_kind, error);
 
-	/* write */
-	blob = fu_firmware_write(self, error);
-	if (blob == NULL)
-		return NULL;
-	return g_compute_checksum_for_bytes(csum_kind, blob);
+	/* nothing to do */
+	g_set_error_literal(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_FOUND,
+			    "no input data, as no FuFirmware->write, stream or bytes");
+	return NULL;
 }
 
 /**
@@ -1002,11 +971,13 @@ fu_firmware_check_compatible(FuFirmware *self,
 static gboolean
 fu_firmware_validate_for_offset(FuFirmware *self,
 				GInputStream *stream,
-				gsize *offset,
+				gsize offset,
+				gsize *offset_found,
 				FuFirmwareParseFlags flags,
 				GError **error)
 {
 	FuFirmwareClass *klass = FU_FIRMWARE_GET_CLASS(self);
+	FuFirmwarePrivate *priv = GET_PRIVATE(self);
 	gsize streamsz = 0;
 
 	/* not implemented */
@@ -1016,38 +987,50 @@ fu_firmware_validate_for_offset(FuFirmware *self,
 	/* fuzzing */
 	if (!fu_firmware_has_flag(self, FU_FIRMWARE_FLAG_ALWAYS_SEARCH) &&
 	    (flags & FU_FIRMWARE_PARSE_FLAG_NO_SEARCH) > 0) {
-		if (!klass->validate(self, stream, *offset, error))
-			return FALSE;
-		return TRUE;
+		return klass->validate(self, stream, offset, error);
 	}
 
-	/* limit the size of firmware we search */
+	/* try all the magic values, if provided */
+	if (priv->magic != NULL) {
+		for (guint i = 0; i < priv->magic->len; i++) {
+			FuFirmwarePatch *patch = g_ptr_array_index(priv->magic, i);
+			gsize offset_tmp = 0;
+			g_debug("searching for 0x%x bytes of magic",
+				(guint)g_bytes_get_size(patch->blob));
+			if (fu_input_stream_find(stream,
+						 g_bytes_get_data(patch->blob, NULL),
+						 g_bytes_get_size(patch->blob),
+						 offset,
+						 &offset_tmp,
+						 NULL)) {
+				offset_tmp -= patch->offset;
+				g_debug("found magic @0x%x", (guint)offset_tmp);
+				if (offset_found != NULL)
+					*offset_found = offset_tmp;
+				return klass->validate(self, stream, *offset_found, error);
+			}
+		}
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INVALID_FILE,
+				    "failed to find magic bytes");
+		return FALSE;
+	}
+
+	/* limit the size of firmware we search as brute force is expensive */
 	if (!fu_input_stream_size(stream, &streamsz, error))
 		return FALSE;
-	if (streamsz > FU_FIRMWARE_SEARCH_MAGIC_BUFSZ_MAX) {
-		if (!klass->validate(self, stream, *offset, error)) {
-			g_prefix_error(error,
-				       "failed to search for magic as firmware size was 0x%x and "
-				       "limit was 0x%x: ",
-				       (guint)streamsz,
-				       (guint)FU_FIRMWARE_SEARCH_MAGIC_BUFSZ_MAX);
-			return FALSE;
-		}
-		return TRUE;
-	}
-
-	/* increment the offset, looking for the magic */
-	for (gsize offset_tmp = *offset; offset_tmp < streamsz; offset_tmp++) {
-		if (klass->validate(self, stream, offset_tmp, NULL)) {
-			fu_firmware_set_offset(self, offset_tmp);
-			*offset = offset_tmp;
-			return TRUE;
+	if (streamsz < FU_FIRMWARE_SEARCH_MAGIC_BUFSZ_MAX) {
+		for (gsize offset_tmp = offset; offset_tmp < streamsz; offset_tmp++) {
+			if (klass->validate(self, stream, offset_tmp, NULL)) {
+				*offset_found = offset_tmp;
+				return TRUE;
+			}
 		}
 	}
 
-	/* did not find what we were looking for */
-	g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_INVALID_FILE, "did not find magic");
-	return FALSE;
+	/* check in more detail */
+	return klass->validate(self, stream, offset, error);
 }
 
 /**
@@ -1075,6 +1058,8 @@ fu_firmware_parse_stream(FuFirmware *self,
 	FuFirmwarePrivate *priv = GET_PRIVATE(self);
 	gsize streamsz = 0;
 	g_autoptr(GInputStream) partial_stream = NULL;
+	g_autoptr(GInputStream) seekable_stream = NULL;
+	g_autoptr(GBytes) blob = NULL;
 
 	g_return_val_if_fail(FU_IS_FIRMWARE(self), FALSE);
 	g_return_val_if_fail(G_IS_INPUT_STREAM(stream), FALSE);
@@ -1089,8 +1074,18 @@ fu_firmware_parse_stream(FuFirmware *self,
 		return FALSE;
 	}
 
+	/* ensure the stream is seekable */
+	if (!G_IS_SEEKABLE(stream) || !g_seekable_can_seek(G_SEEKABLE(stream))) {
+		blob = fu_input_stream_read_bytes(stream, offset, G_MAXUINT32, NULL, error);
+		if (blob == NULL)
+			return FALSE;
+		seekable_stream = g_memory_input_stream_new_from_bytes(blob);
+	} else {
+		seekable_stream = g_object_ref(stream);
+	}
+
 	/* check size */
-	if (!fu_input_stream_size(stream, &streamsz, error))
+	if (!fu_input_stream_size(seekable_stream, &streamsz, error))
 		return FALSE;
 	if (streamsz <= offset) {
 		g_set_error(error,
@@ -1103,8 +1098,9 @@ fu_firmware_parse_stream(FuFirmware *self,
 	}
 
 	/* optional */
-	if (!fu_firmware_validate_for_offset(self, stream, &offset, flags, error))
+	if (!fu_firmware_validate_for_offset(self, seekable_stream, offset, &offset, flags, error))
 		return FALSE;
+	fu_firmware_set_offset(self, offset);
 
 	/* save stream size */
 	priv->streamsz = streamsz - offset;
@@ -1137,26 +1133,31 @@ fu_firmware_parse_stream(FuFirmware *self,
 
 	/* save stream */
 	if (offset == 0) {
-		partial_stream = g_object_ref(stream);
+		partial_stream = g_object_ref(seekable_stream);
 	} else {
-		partial_stream = fu_partial_input_stream_new(stream, offset, priv->streamsz, error);
+		partial_stream =
+		    fu_partial_input_stream_new(seekable_stream, offset, priv->streamsz, error);
 		if (partial_stream == NULL) {
-			g_prefix_error(error, "failed to cut firmware: ");
+			g_prefix_error_literal(error, "failed to cut firmware: ");
 			return FALSE;
 		}
 	}
 
 	/* cache */
 	if (flags & FU_FIRMWARE_PARSE_FLAG_CACHE_BLOB) {
-		g_autoptr(GBytes) blob = NULL;
-		blob = fu_input_stream_read_bytes(partial_stream, 0x0, priv->streamsz, NULL, error);
-		if (blob == NULL)
-			return FALSE;
+		if (blob == NULL) {
+			blob = fu_input_stream_read_bytes(partial_stream,
+							  0x0,
+							  priv->streamsz,
+							  NULL,
+							  error);
+			if (blob == NULL)
+				return FALSE;
+		}
 		fu_firmware_set_bytes(self, blob);
 	}
-	if (flags & FU_FIRMWARE_PARSE_FLAG_CACHE_STREAM) {
+	if (flags & FU_FIRMWARE_PARSE_FLAG_CACHE_STREAM)
 		g_set_object(&priv->stream, partial_stream);
-	}
 
 	/* optional */
 	if (klass->tokenize != NULL) {
@@ -1165,7 +1166,9 @@ fu_firmware_parse_stream(FuFirmware *self,
 	}
 
 	/* optional */
-	if (klass->parse != NULL)
+	if (klass->parse_full != NULL)
+		return klass->parse_full(self, seekable_stream, offset, flags, error);
+	else if (klass->parse != NULL)
 		return klass->parse(self, partial_stream, flags, error);
 
 	/* verify alignment */
@@ -1284,7 +1287,7 @@ fu_firmware_build(FuFirmware *self, XbNode *n, GError **error)
 	/* set attributes */
 	tmp = xb_node_query_text(n, "version", NULL);
 	if (tmp != NULL)
-		fu_firmware_set_version(self, tmp);
+		fu_firmware_set_version(self, tmp); /* nocheck:set-version */
 	tmp = xb_node_query_text(n, "version_format", NULL);
 	if (tmp != NULL) {
 		FwupdVersionFormat version_format = fwupd_version_format_from_string(tmp);
@@ -1399,7 +1402,7 @@ fu_firmware_build(FuFirmware *self, XbNode *n, GError **error)
 			} else {
 				img = fu_firmware_new();
 			}
-			if (!fu_firmware_add_image_full(self, img, error))
+			if (!fu_firmware_add_image(self, img, error))
 				return FALSE;
 			if (!fu_firmware_build(img, xb_image, error))
 				return FALSE;
@@ -1476,7 +1479,7 @@ fu_firmware_build_from_xml(FuFirmware *self, const gchar *xml, GError **error)
 
 	/* parse XML */
 	if (!xb_builder_source_load_xml(source, xml, XB_BUILDER_SOURCE_FLAG_NONE, error)) {
-		g_prefix_error(error, "could not parse XML: ");
+		g_prefix_error_literal(error, "could not parse XML: ");
 		fwupd_error_convert(error);
 		return FALSE;
 	}
@@ -1546,7 +1549,7 @@ fu_firmware_parse_file(FuFirmware *self, GFile *file, FuFirmwareParseFlags flags
 
 	stream = g_file_read(file, NULL, error);
 	if (stream == NULL) {
-		fu_error_convert(error);
+		fwupd_error_convert(error);
 		return FALSE;
 	}
 	return fu_firmware_parse_stream(self, G_INPUT_STREAM(stream), 0, flags, error);
@@ -1681,9 +1684,8 @@ fu_firmware_write_chunk(FuFirmware *self, guint64 address, guint64 chunk_sz_max,
 
 	/* if we have less data than requested */
 	chunk_left = g_bytes_get_size(priv->bytes) - offset;
-	if (chunk_sz_max > chunk_left) {
+	if (chunk_sz_max > chunk_left)
 		return fu_bytes_new_offset(priv->bytes, offset, chunk_left, error);
-	}
 
 	/* check chunk */
 	return fu_bytes_new_offset(priv->bytes, offset, chunk_sz_max, error);
@@ -1757,7 +1759,7 @@ fu_firmware_get_depth(FuFirmware *self)
 }
 
 /**
- * fu_firmware_add_image_full:
+ * fu_firmware_add_image:
  * @self: a #FuPlugin
  * @img: a child firmware image
  * @error: (nullable): optional return location for an error
@@ -1770,10 +1772,10 @@ fu_firmware_get_depth(FuFirmware *self)
  *
  * Returns: %TRUE if the image was added
  *
- * Since: 1.9.3
+ * Since: 1.9.3, but @error was added in 2.0.17
  **/
 gboolean
-fu_firmware_add_image_full(FuFirmware *self, FuFirmware *img, GError **error)
+fu_firmware_add_image(FuFirmware *self, FuFirmware *img, GError **error)
 {
 	FuFirmwarePrivate *priv = GET_PRIVATE(self);
 
@@ -1792,15 +1794,18 @@ fu_firmware_add_image_full(FuFirmware *self, FuFirmware *img, GError **error)
 	}
 
 	/* dedupe */
-	for (guint i = 0; i < priv->images->len; i++) {
-		FuFirmware *img_tmp = g_ptr_array_index(priv->images, i);
-		if (priv->flags & FU_FIRMWARE_FLAG_DEDUPE_ID) {
+	if (priv->flags & FU_FIRMWARE_FLAG_DEDUPE_ID) {
+		for (guint i = 0; i < priv->images->len; i++) {
+			FuFirmware *img_tmp = g_ptr_array_index(priv->images, i);
 			if (g_strcmp0(fu_firmware_get_id(img_tmp), fu_firmware_get_id(img)) == 0) {
 				g_ptr_array_remove_index(priv->images, i);
 				break;
 			}
 		}
-		if (priv->flags & FU_FIRMWARE_FLAG_DEDUPE_IDX) {
+	}
+	if (priv->flags & FU_FIRMWARE_FLAG_DEDUPE_IDX) {
+		for (guint i = 0; i < priv->images->len; i++) {
+			FuFirmware *img_tmp = g_ptr_array_index(priv->images, i);
 			if (fu_firmware_get_idx(img_tmp) == fu_firmware_get_idx(img)) {
 				g_ptr_array_remove_index(priv->images, i);
 				break;
@@ -1828,33 +1833,6 @@ fu_firmware_add_image_full(FuFirmware *self, FuFirmware *img, GError **error)
 	return TRUE;
 }
 
-/**
- * fu_firmware_add_image:
- * @self: a #FuPlugin
- * @img: a child firmware image
- *
- * Adds an image to the firmware.
- *
- * NOTE: If adding images in a loop of any kind then fu_firmware_add_image_full() should be used
- * instead, and fu_firmware_set_images_max() should be set before adding images.
- *
- * If %FU_FIRMWARE_FLAG_DEDUPE_ID is set, an image with the same ID is already
- * present it is replaced.
- *
- * Since: 1.3.1
- **/
-void
-fu_firmware_add_image(FuFirmware *self, FuFirmware *img)
-{
-	g_autoptr(GError) error_local = NULL;
-
-	g_return_if_fail(FU_IS_FIRMWARE(self));
-	g_return_if_fail(FU_IS_FIRMWARE(img));
-
-	if (!fu_firmware_add_image_full(self, img, &error_local))
-		g_critical("failed to add image: %s", error_local->message);
-}
-
 /**
  * fu_firmware_set_images_max:
  * @self: a #FuPlugin
@@ -2021,38 +1999,53 @@ FuFirmware *
 fu_firmware_get_image_by_id(FuFirmware *self, const gchar *id, GError **error)
 {
 	FuFirmwarePrivate *priv = GET_PRIVATE(self);
+	g_autofree gchar *id_str = NULL;
+	g_autoptr(GPtrArray) id_errmsg = g_ptr_array_new();
 
 	g_return_val_if_fail(FU_IS_FIRMWARE(self), NULL);
 	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
+	/* sanity check */
+	if (priv->images->len == 0) {
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_NOT_FOUND,
+				    "no images in firmware");
+		return NULL;
+	}
+
 	/* non-NULL */
 	if (id != NULL) {
 		g_auto(GStrv) split = g_strsplit(id, "|", 0);
 		for (guint i = 0; i < priv->images->len; i++) {
 			FuFirmware *img = g_ptr_array_index(priv->images, i);
 			for (guint j = 0; split[j] != NULL; j++) {
+				if (fu_firmware_get_id(img) == NULL)
+					continue;
 				if (g_pattern_match_simple(split[j], fu_firmware_get_id(img)))
 					return g_object_ref(img);
 			}
 		}
-		g_set_error(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_FOUND,
-			    "no image id %s found in firmware",
-			    id);
-		return NULL;
+	} else {
+		for (guint i = 0; i < priv->images->len; i++) {
+			FuFirmware *img = g_ptr_array_index(priv->images, i);
+			if (fu_firmware_get_id(img) == NULL)
+				return g_object_ref(img);
+		}
 	}
 
-	/* NULL */
+	/* build a useful error */
 	for (guint i = 0; i < priv->images->len; i++) {
 		FuFirmware *img = g_ptr_array_index(priv->images, i);
-		if (fu_firmware_get_id(img) == NULL)
-			return g_object_ref(img);
+		g_ptr_array_add(id_errmsg, (gpointer)fu_firmware_get_id(img));
 	}
-	g_set_error_literal(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_FOUND,
-			    "no NULL image id found in firmware");
+	id_str = fu_strjoin(",", id_errmsg);
+	g_set_error(error,
+		    FWUPD_ERROR,
+		    FWUPD_ERROR_NOT_FOUND,
+		    "no image id %s found in firmware, only have %s",
+		    id,
+		    id_str);
 	return NULL;
 }
 
@@ -2272,6 +2265,40 @@ fu_firmware_get_image_by_gtype(FuFirmware *self, GType gtype, GError **error)
 	return NULL;
 }
 
+static gint
+fu_firmware_sort_by_id_cb(gconstpointer a, gconstpointer b)
+{
+	FuFirmware *firmware1 = *((FuFirmware **)a);
+	FuFirmware *firmware2 = *((FuFirmware **)b);
+	return g_strcmp0(fu_firmware_get_id(firmware1), fu_firmware_get_id(firmware2));
+}
+
+static gint
+fu_firmware_sort_by_idx_cb(gconstpointer a, gconstpointer b)
+{
+	FuFirmware *firmware1 = *((FuFirmware **)a);
+	FuFirmware *firmware2 = *((FuFirmware **)b);
+	guint64 idx1 = fu_firmware_get_idx(firmware1);
+	guint64 idx2 = fu_firmware_get_idx(firmware2);
+	if (idx1 < idx2)
+		return -1;
+	if (idx1 > idx2)
+		return 1;
+	return 0;
+}
+
+static GPtrArray *
+fu_firmware_get_images_sorted(FuFirmware *self)
+{
+	FuFirmwarePrivate *priv = GET_PRIVATE(self);
+	g_autoptr(GPtrArray) images = g_ptr_array_copy(priv->images, (GCopyFunc)g_object_ref, NULL);
+	if (priv->flags & FU_FIRMWARE_FLAG_DEDUPE_IDX)
+		g_ptr_array_sort(images, fu_firmware_sort_by_idx_cb);
+	if (priv->flags & FU_FIRMWARE_FLAG_DEDUPE_ID)
+		g_ptr_array_sort(images, fu_firmware_sort_by_id_cb);
+	return g_steal_pointer(&images);
+}
+
 /**
  * fu_firmware_export:
  * @self: a #FuFirmware
@@ -2280,6 +2307,10 @@ fu_firmware_get_image_by_gtype(FuFirmware *self, GType gtype, GError **error)
  *
  * This allows us to build an XML object for the nested firmware.
  *
+ * The image children will have a predictable order if the firmware has
+ * %FU_FIRMWARE_FLAG_DEDUPE_ID or %FU_FIRMWARE_FLAG_DEDUPE_IDX and
+ * %FU_FIRMWARE_EXPORT_FLAG_SORTED is also used in @flags.
+ *
  * Since: 1.6.0
  **/
 void
@@ -2295,19 +2326,8 @@ fu_firmware_export(FuFirmware *self, FuFirmwareExportFlags flags, XbBuilderNode
 
 	/* subclassed type */
 	if (priv->flags != FU_FIRMWARE_FLAG_NONE) {
-		g_autoptr(GString) tmp = g_string_new("");
-		for (guint i = 0; i < 64; i++) {
-			guint64 flag = (guint64)1 << i;
-			if (flag == FU_FIRMWARE_FLAG_DONE_PARSE)
-				continue;
-			if ((priv->flags & flag) == 0)
-				continue;
-			g_string_append_printf(tmp, "%s|", fu_firmware_flag_to_string(flag));
-		}
-		if (tmp->len > 0) {
-			g_string_truncate(tmp, tmp->len - 1);
-			fu_xmlb_builder_insert_kv(bn, "flags", tmp->str);
-		}
+		g_autofree gchar *str = fu_firmware_flags_to_string(priv->flags);
+		fu_xmlb_builder_insert_kv(bn, "flags", str);
 	}
 	fu_xmlb_builder_insert_kv(bn, "id", priv->id);
 	fu_xmlb_builder_insert_kx(bn, "idx", priv->idx);
@@ -2385,14 +2405,28 @@ fu_firmware_export(FuFirmware *self, FuFirmwareExportFlags flags, XbBuilderNode
 		}
 	}
 
+	/* magic */
+	if (priv->magic != NULL && priv->magic->len > 0) {
+		g_autoptr(XbBuilderNode) bp = xb_builder_node_insert(bn, "magic", NULL);
+		for (guint i = 0; i < priv->magic->len; i++) {
+			FuFirmwarePatch *patch = g_ptr_array_index(priv->magic, i);
+			g_autofree gchar *str = fu_bytes_to_string(patch->blob);
+			g_autofree gchar *offset = g_strdup_printf("0x%x", (guint)patch->offset);
+			xb_builder_node_insert_text(bp, "data", str, "offset", offset, NULL);
+		}
+	}
+
 	/* vfunc */
 	if (klass->export != NULL)
 		klass->export(self, flags, bn);
 
 	/* children */
 	if (priv->images->len > 0) {
-		for (guint i = 0; i < priv->images->len; i++) {
-			FuFirmware *img = g_ptr_array_index(priv->images, i);
+		g_autoptr(GPtrArray) images = flags & FU_FIRMWARE_EXPORT_FLAG_SORTED
+						  ? fu_firmware_get_images_sorted(self)
+						  : g_ptr_array_ref(priv->images);
+		for (guint i = 0; i < images->len; i++) {
+			FuFirmware *img = g_ptr_array_index(images, i);
 			g_autoptr(XbBuilderNode) bc = xb_builder_node_insert(bn, "firmware", NULL);
 			fu_firmware_export(img, flags, bc);
 		}
@@ -2484,6 +2518,15 @@ fu_firmware_init(FuFirmware *self)
 	priv->images = g_ptr_array_new_with_free_func((GDestroyNotify)g_object_unref);
 }
 
+static void
+fu_firmware_constructed(GObject *obj)
+{
+	FuFirmware *self = FU_FIRMWARE(obj);
+	FuFirmwareClass *klass = FU_FIRMWARE_GET_CLASS(self);
+	if (klass->add_magic != NULL)
+		klass->add_magic(self);
+}
+
 static void
 fu_firmware_finalize(GObject *object)
 {
@@ -2500,6 +2543,8 @@ fu_firmware_finalize(GObject *object)
 		g_ptr_array_unref(priv->chunks);
 	if (priv->patches != NULL)
 		g_ptr_array_unref(priv->patches);
+	if (priv->magic != NULL)
+		g_ptr_array_unref(priv->magic);
 	if (priv->parent != NULL)
 		g_object_remove_weak_pointer(G_OBJECT(priv->parent), (gpointer *)&priv->parent);
 	g_ptr_array_unref(priv->images);
@@ -2515,6 +2560,7 @@ fu_firmware_class_init(FuFirmwareClass *klass)
 	object_class->finalize = fu_firmware_finalize;
 	object_class->get_property = fu_firmware_get_property;
 	object_class->set_property = fu_firmware_set_property;
+	object_class->constructed = fu_firmware_constructed;
 
 	/**
 	 * FuFirmware:parent:
@@ -2618,10 +2664,11 @@ fu_firmware_new_from_gtypes(GInputStream *stream,
 		g_autoptr(FuFirmware) firmware = g_object_new(gtype, NULL);
 		g_autoptr(GError) error_local = NULL;
 		if (!fu_firmware_parse_stream(firmware, stream, offset, flags, &error_local)) {
-			g_debug("%s", error_local->message);
+			g_debug("@0x%x %s", (guint)offset, error_local->message);
 			if (error_all == NULL) {
 				g_propagate_error(&error_all, g_steal_pointer(&error_local));
 			} else {
+				/* nocheck:error */
 				g_prefix_error(&error_all, "%s: ", error_local->message);
 			}
 			continue;
