@@ -16,9 +16,10 @@ enum FuIfdRegion {
     Max = 0x0F,
 }
 
-#[derive(ParseStream, New, ValidateStream)]
+#[derive(ParseStream, New, ValidateStream, Default)]
+#[repr(C, packed)]
 struct FuStructIfdFdbar {
-    reserved: [u8; 16] = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF,
+    reserved: [u8; 16] = [0xFF; 16],
     signature: u32le == 0x0FF0A55A,
     descriptor_map0: u32le,
     descriptor_map1: u32le,
@@ -26,8 +27,17 @@ struct FuStructIfdFdbar {
 }
 
 #[derive(ParseStream, New)]
+#[repr(C, packed)]
 struct FuStructIfdFcba {
     flcomp: u32le,
     flill: u32le,
     flill1: u32le,
 }
+
+#[derive(ToString)]
+#[repr(C, packed)]
+enum FuIfdAccess {
+    None = 0,
+    Read = 1 << 0,
+    Write = 1 << 1,
+}
