@@ -22,6 +22,8 @@
 struct _FuDellDockStatus {
 	FuDevice parent_instance;
 	guint64 blob_version_offset;
+	guint8 dock_type;
+	gboolean dock_usb4_present;
 };
 
 G_DEFINE_TYPE(FuDellDockStatus, fu_dell_dock_status, FU_TYPE_DEVICE)
@@ -44,8 +46,10 @@ fu_dell_dock_status_setup(FuDevice *device, GError **error)
 	guint32 status_version;
 	g_autofree gchar *dynamic_version = NULL;
 
-	parent = fu_device_get_parent(device);
-	status_version = fu_dell_dock_ec_get_status_version(parent);
+	parent = fu_device_get_parent(device, error);
+	if (parent == NULL)
+		return FALSE;
+	status_version = fu_dell_dock_ec_get_status_version(FU_DELL_DOCK_EC(parent));
 
 	dynamic_version = fu_dell_dock_status_ver_string(status_version);
 	fu_device_set_version_format(device, FWUPD_VERSION_FORMAT_QUAD);
@@ -62,6 +66,7 @@ fu_dell_dock_status_write(FuDevice *device,
 			  GError **error)
 {
 	FuDellDockStatus *self = FU_DELL_DOCK_STATUS(device);
+	FuDellDockEc *proxy;
 	gsize length = 0;
 	guint32 status_version = 0;
 	const guint8 *data;
@@ -88,7 +93,10 @@ fu_dell_dock_status_write(FuDevice *device,
 	dynamic_version = fu_dell_dock_status_ver_string(status_version);
 	g_info("writing status firmware version %s", dynamic_version);
 
-	if (!fu_dell_dock_ec_commit_package(fu_device_get_proxy(device), fw, error))
+	proxy = FU_DELL_DOCK_EC(fu_device_get_proxy(FU_DEVICE(self), error));
+	if (proxy == NULL)
+		return FALSE;
+	if (!fu_dell_dock_ec_commit_package(proxy, fw, error))
 		return FALSE;
 
 	/* dock will reboot to re-read; this is to appease the daemon */
@@ -97,27 +105,53 @@ fu_dell_dock_status_write(FuDevice *device,
 	return TRUE;
 }
 
+static gboolean
+fu_dell_dock_status_probe(FuDevice *device, GError **error)
+{
+	FuDellDockStatus *self = FU_DELL_DOCK_STATUS(device);
+
+	if (self->dock_type == DOCK_BASE_TYPE_ATOMIC) {
+		fu_device_add_instance_id(device, DELL_DOCK_ATOMIC_STATUS_INSTANCE_ID);
+	} else if (self->dock_type == DOCK_BASE_TYPE_SALOMON) {
+		if (self->dock_usb4_present)
+			fu_device_add_instance_id(device, DELL_DOCK_DOCK2_INSTANCE_ID);
+		else
+			fu_device_add_instance_id(device, DELL_DOCK_DOCK1_INSTANCE_ID);
+	} else {
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
+			    "unknown supported dock type 0x%x",
+			    self->dock_type);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 fu_dell_dock_status_open(FuDevice *device, GError **error)
 {
-	if (fu_device_get_proxy(device) == NULL) {
-		if (fu_device_get_parent(device) == NULL) {
-			g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_INTERNAL, "no parent");
+	FuDevice *proxy;
+
+	proxy = fu_device_get_proxy(device, NULL);
+	if (proxy == NULL) {
+		proxy = fu_device_get_parent(device, error);
+		if (proxy == NULL)
 			return FALSE;
-		}
-		fu_device_set_proxy(device, fu_device_get_parent(device));
+		fu_device_set_proxy(device, proxy);
 	}
-	return fu_device_open(fu_device_get_proxy(device), error);
+	return fu_device_open(proxy, error);
 }
 
 static gboolean
 fu_dell_dock_status_close(FuDevice *device, GError **error)
 {
-	if (fu_device_get_proxy(device) == NULL) {
-		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_INTERNAL, "no proxy");
+	FuDevice *proxy;
+	proxy = fu_device_get_proxy(device, error);
+	if (proxy == NULL)
 		return FALSE;
-	}
-	return fu_device_close(fu_device_get_proxy(device), error);
+	return fu_device_close(proxy, error);
 }
 
 static gboolean
@@ -144,9 +178,10 @@ fu_dell_dock_status_set_quirk_kv(FuDevice *device,
 }
 
 static void
-fu_dell_dock_status_set_progress(FuDevice *self, FuProgress *progress)
+fu_dell_dock_status_set_progress(FuDevice *device, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 13, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 72, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 9, "attach");
@@ -159,6 +194,7 @@ fu_dell_dock_status_init(FuDellDockStatus *self)
 	fu_device_add_protocol(FU_DEVICE(self), "com.dell.dock");
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UNSIGNED_PAYLOAD);
+	fu_device_set_proxy_gtype(FU_DEVICE(self), FU_TYPE_HID_DEVICE);
 }
 
 static void
@@ -166,6 +202,7 @@ fu_dell_dock_status_class_init(FuDellDockStatusClass *klass)
 {
 	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
 	device_class->write_firmware = fu_dell_dock_status_write;
+	device_class->probe = fu_dell_dock_status_probe;
 	device_class->setup = fu_dell_dock_status_setup;
 	device_class->open = fu_dell_dock_status_open;
 	device_class->close = fu_dell_dock_status_close;
@@ -174,9 +211,11 @@ fu_dell_dock_status_class_init(FuDellDockStatusClass *klass)
 }
 
 FuDellDockStatus *
-fu_dell_dock_status_new(FuContext *ctx)
+fu_dell_dock_status_new(FuContext *ctx, guint8 dock_type, gboolean dock_usb4_present)
 {
 	FuDellDockStatus *self = NULL;
 	self = g_object_new(FU_TYPE_DELL_DOCK_STATUS, "context", ctx, NULL);
+	self->dock_type = dock_type;
+	self->dock_usb4_present = dock_usb4_present;
 	return self;
 }
