@@ -10,7 +10,6 @@
 #include "fu-igsc-aux-device.h"
 #include "fu-igsc-aux-firmware.h"
 #include "fu-igsc-device.h"
-#include "fu-igsc-heci.h"
 
 struct _FuIgscAuxDevice {
 	FuDevice parent_instance;
@@ -33,22 +32,24 @@ fu_igsc_aux_device_to_string(FuDevice *device, guint idt, GString *str)
 static gboolean
 fu_igsc_aux_device_probe(FuDevice *device, GError **error)
 {
-	FuDevice *parent = fu_device_get_parent(device);
+	FuDevice *parent;
 
-	/* fix name */
-	if (parent != NULL) {
-		g_autofree gchar *name = NULL;
-		name = g_strdup_printf("%s Data", fu_device_get_name(parent));
-		fu_device_set_name(device, name);
-	}
+	/* from the self tests */
+	parent = fu_device_get_parent(device, error);
+	if (parent == NULL)
+		return FALSE;
 
 	/* add extra instance IDs */
-	fu_device_add_instance_str(device, "PART", "FWDATA");
-	if (!fu_device_build_instance_id(device, error, "MEI", "VEN", "DEV", "PART", NULL))
+	fu_device_add_instance_str(device,
+				   "PART",
+				   fu_device_has_private_flag(parent, FU_IGSC_DEVICE_FLAG_IS_WEDGED)
+				       ? "FWDATA_RECOVERY"
+				       : "FWDATA");
+	if (!fu_device_build_instance_id(device, error, "PCI", "VEN", "DEV", "PART", NULL))
 		return FALSE;
 	return fu_device_build_instance_id(device,
 					   error,
-					   "MEI",
+					   "PCI",
 					   "VEN",
 					   "DEV",
 					   "SUBSYS",
@@ -60,19 +61,26 @@ static gboolean
 fu_igsc_aux_device_setup(FuDevice *device, GError **error)
 {
 	FuIgscAuxDevice *self = FU_IGSC_AUX_DEVICE(device);
-	FuIgscDevice *igsc_parent = FU_IGSC_DEVICE(fu_device_get_parent(device));
+	FuIgscDevice *parent;
 	g_autofree gchar *version = NULL;
 
 	/* get version */
-	if (!fu_igsc_device_get_aux_version(igsc_parent,
+	parent = FU_IGSC_DEVICE(fu_device_get_parent(device, error));
+	if (parent == NULL)
+		return FALSE;
+	if (!fu_igsc_device_get_aux_version(parent,
 					    &self->oem_version,
 					    &self->major_version,
 					    &self->major_vcn,
 					    error)) {
-		g_prefix_error(error, "failed to get aux version: ");
+		g_prefix_error_literal(error, "failed to get aux version: ");
 		return FALSE;
 	}
-	version = g_strdup_printf("%u.%x", self->major_version, self->oem_version);
+	if (fu_device_has_private_flag(FU_DEVICE(parent), FU_IGSC_DEVICE_FLAG_IS_WEDGED)) {
+		version = g_strdup("0.0");
+	} else {
+		version = g_strdup_printf("%u.%u", self->major_version, self->oem_version);
+	}
 	fu_device_set_version(device, version);
 
 	/* success */
@@ -87,7 +95,7 @@ fu_igsc_aux_device_prepare_firmware(FuDevice *device,
 				    GError **error)
 {
 	FuIgscAuxDevice *self = FU_IGSC_AUX_DEVICE(device);
-	FuIgscDevice *igsc_parent = FU_IGSC_DEVICE(fu_device_get_parent(device));
+	FuIgscDevice *parent;
 	g_autoptr(FuIgscAuxFirmware) firmware = FU_IGSC_AUX_FIRMWARE(fu_igsc_aux_firmware_new());
 
 	/* parse container */
@@ -95,11 +103,14 @@ fu_igsc_aux_device_prepare_firmware(FuDevice *device,
 		return NULL;
 
 	/* search the device list for a match */
+	parent = FU_IGSC_DEVICE(fu_device_get_parent(device, error));
+	if (parent == NULL)
+		return NULL;
 	if (!fu_igsc_aux_firmware_match_device(firmware,
-					       self->major_version,
-					       self->major_vcn,
-					       fu_igsc_device_get_ssvid(igsc_parent),
-					       fu_igsc_device_get_ssdid(igsc_parent),
+					       fu_device_get_vid(FU_DEVICE(parent)),
+					       fu_device_get_pid(FU_DEVICE(parent)),
+					       fu_igsc_device_get_ssvid(parent),
+					       fu_igsc_device_get_ssdid(parent),
 					       error))
 		return NULL;
 
@@ -126,7 +137,7 @@ fu_igsc_aux_device_prepare_firmware(FuDevice *device,
 		g_set_error(error,
 			    FWUPD_ERROR,
 			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "invalid manufacturer data version, got 0x%x, expected 0x%x",
+			    "invalid OEM version, got 0x%x, expected higher than 0x%x",
 			    fu_igsc_aux_firmware_get_oem_version(firmware),
 			    self->oem_version);
 		return NULL;
@@ -143,7 +154,7 @@ fu_igsc_aux_device_write_firmware(FuDevice *device,
 				  FwupdInstallFlags flags,
 				  GError **error)
 {
-	FuIgscDevice *igsc_parent = FU_IGSC_DEVICE(fu_device_get_parent(device));
+	FuIgscDevice *parent;
 	g_autoptr(GBytes) fw_info = NULL;
 	g_autoptr(GInputStream) stream_payload = NULL;
 
@@ -156,7 +167,10 @@ fu_igsc_aux_device_write_firmware(FuDevice *device,
 	    fu_firmware_get_image_by_idx_stream(firmware, FU_IFWI_FPT_FIRMWARE_IDX_SDTA, error);
 	if (stream_payload == NULL)
 		return FALSE;
-	return fu_igsc_device_write_blob(igsc_parent,
+	parent = FU_IGSC_DEVICE(fu_device_get_parent(device, error));
+	if (parent == NULL)
+		return FALSE;
+	return fu_igsc_device_write_blob(parent,
 					 FU_IGSC_FWU_HECI_PAYLOAD_TYPE_FWDATA,
 					 fw_info,
 					 stream_payload,
@@ -164,53 +178,48 @@ fu_igsc_aux_device_write_firmware(FuDevice *device,
 					 error);
 }
 
-static gboolean
-fu_igsc_aux_device_prepare(FuDevice *device,
-			   FuProgress *progress,
-			   FwupdInstallFlags flags,
-			   GError **error)
+static void
+fu_igsc_aux_device_set_progress(FuDevice *device, FuProgress *progress)
 {
-	/* set PCI power policy */
-	return fu_device_prepare(fu_device_get_parent(device), progress, flags, error);
-}
-
-static gboolean
-fu_igsc_aux_device_cleanup(FuDevice *device,
-			   FuProgress *progress,
-			   FwupdInstallFlags flags,
-			   GError **error)
-{
-	/* set PCI power policy */
-	return fu_device_cleanup(fu_device_get_parent(device), progress, flags, error);
+	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_flag(progress, FU_PROGRESS_FLAG_GUESSED);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "detach");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 100, "write");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "attach");
+	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_BUSY, 0, "reload");
 }
 
 static void
 fu_igsc_aux_device_init(FuIgscAuxDevice *self)
 {
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_REQUIRE_AC);
+	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_NEEDS_REBOOT);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_INTERNAL);
+	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_ONLY_VERSION_UPGRADE);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_PARENT_NAME_PREFIX);
+	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_USE_PROXY_FOR_OPEN);
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_PAIR);
+	fu_device_set_proxy_gtype(FU_DEVICE(self), FU_TYPE_IGSC_DEVICE);
 	fu_device_add_protocol(FU_DEVICE(self), "com.intel.gsc");
 	fu_device_set_logical_id(FU_DEVICE(self), "fw-data");
+	fu_device_set_name(FU_DEVICE(self), "Data");
 }
 
 static void
 fu_igsc_aux_device_class_init(FuIgscAuxDeviceClass *klass)
 {
 	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
+	device_class->set_progress = fu_igsc_aux_device_set_progress;
 	device_class->to_string = fu_igsc_aux_device_to_string;
 	device_class->probe = fu_igsc_aux_device_probe;
 	device_class->setup = fu_igsc_aux_device_setup;
 	device_class->prepare_firmware = fu_igsc_aux_device_prepare_firmware;
 	device_class->write_firmware = fu_igsc_aux_device_write_firmware;
-	device_class->prepare = fu_igsc_aux_device_prepare;
-	device_class->cleanup = fu_igsc_aux_device_cleanup;
 }
 
 FuIgscAuxDevice *
-fu_igsc_aux_device_new(FuContext *ctx)
+fu_igsc_aux_device_new(FuDevice *proxy)
 {
-	return g_object_new(FU_TYPE_IGSC_AUX_DEVICE, "context", ctx, NULL);
+	return g_object_new(FU_TYPE_IGSC_AUX_DEVICE, "proxy", proxy, NULL);
 }
