@@ -22,8 +22,8 @@ G_DEFINE_TYPE(FuLogitechRallysystemAudioDevice,
 	      FU_TYPE_HIDRAW_DEVICE)
 
 static gboolean
-fu_logitech_rallysystem_audio_device_set_version(FuLogitechRallysystemAudioDevice *self,
-						 GError **error)
+fu_logitech_rallysystem_audio_device_ensure_version(FuLogitechRallysystemAudioDevice *self,
+						    GError **error)
 {
 	guint8 buf[TOPOLOGY_DATA_LEN] = {0x3E, 0x0};
 	guint32 fwversion = 0;
@@ -51,13 +51,13 @@ fu_logitech_rallysystem_audio_device_set_version(FuLogitechRallysystemAudioDevic
 }
 
 static gboolean
-fu_logitech_rallysystem_audio_device_set_serial(FuLogitechRallysystemAudioDevice *self,
-						GError **error)
+fu_logitech_rallysystem_audio_device_ensure_serial(FuLogitechRallysystemAudioDevice *self,
+						   GError **error)
 {
 	guint8 buf_req[SERIAL_NUMBER_REQUEST_DATA_LEN] =
 	    {0x28, 0x85, 0x08, 0xBB, 0x1B, 0x00, 0x01, 0x30, 0, 0, 0, 0x0C};
 	guint8 buf_res[SERIAL_NUMBER_RESPONSE_DATA_LEN] = {0x29, 0x0};
-	g_autoptr(GByteArray) st = NULL;
+	g_autoptr(FuStructAudioSerialNumber) st = NULL;
 	g_autoptr(GString) serial = g_string_new(NULL);
 
 	/* setup HID report for serial number request */
@@ -91,7 +91,7 @@ fu_logitech_rallysystem_audio_device_set_serial(FuLogitechRallysystemAudioDevice
 			       fu_struct_audio_serial_number_get_month(st),
 			       fu_struct_audio_serial_number_get_day(st));
 	for (guint i = 0x0; i < FU_STRUCT_AUDIO_SERIAL_NUMBER_SIZE_MAC_ADDRESS; i++)
-		g_string_append_printf(serial, "%02x", st->data[i]);
+		g_string_append_printf(serial, "%02x", st->buf->data[i]);
 	fu_device_set_serial(FU_DEVICE(self), serial->str);
 
 	/* success */
@@ -102,17 +102,18 @@ static gboolean
 fu_logitech_rallysystem_audio_device_setup(FuDevice *device, GError **error)
 {
 	FuLogitechRallysystemAudioDevice *self = FU_LOGITECH_RALLYSYSTEM_AUDIO_DEVICE(device);
-	if (!fu_logitech_rallysystem_audio_device_set_version(self, error))
+	if (!fu_logitech_rallysystem_audio_device_ensure_version(self, error))
 		return FALSE;
-	if (!fu_logitech_rallysystem_audio_device_set_serial(self, error))
+	if (!fu_logitech_rallysystem_audio_device_ensure_serial(self, error))
 		return FALSE;
 	return TRUE;
 }
 
 static void
-fu_logitech_rallysystem_audio_device_set_progress(FuDevice *self, FuProgress *progress)
+fu_logitech_rallysystem_audio_device_set_progress(FuDevice *device, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 0, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 100, "attach");
