@@ -31,8 +31,8 @@ fu_steelseries_gamepad_cmd_erase(FuSteelseriesGamepad *self, GError **error)
 		/* magic is needed for newer gamepad */
 		fu_struct_steelseries_gamepad_erase_req_set_magic3(st_req, 0x02);
 	}
-	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req, error)) {
-		g_prefix_error(error, "unable erase flash block: ");
+	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req->buf, error)) {
+		g_prefix_error_literal(error, "unable erase flash block: ");
 		return FALSE;
 	}
 
@@ -56,7 +56,7 @@ fu_steelseries_gamepad_setup(FuDevice *device, GError **error)
 		return TRUE;
 
 	/* get version of FW and bootloader */
-	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req, error))
+	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req->buf, error))
 		return FALSE;
 	buf_res = fu_steelseries_device_response(FU_STEELSERIES_DEVICE(self), error);
 	if (buf_res == NULL)
@@ -93,7 +93,7 @@ fu_steelseries_gamepad_attach(FuDevice *device, FuProgress *progress, GError **e
 		return TRUE;
 
 	/* switch to runtime mode */
-	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req, &error_local))
+	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req->buf, &error_local))
 		g_debug("ignoring error on reset: %s", error_local->message);
 
 	fu_device_add_flag(device, FWUPD_DEVICE_FLAG_WAIT_FOR_REPLUG);
@@ -113,7 +113,7 @@ fu_steelseries_gamepad_detach(FuDevice *device, FuProgress *progress, GError **e
 		return TRUE;
 
 	/* switch to bootloader mode */
-	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req, &error_local))
+	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req->buf, &error_local))
 		g_debug("ignoring error on reset: %s", error_local->message);
 
 	/* controller will be renumbered after switching to bootloader mode */
@@ -140,12 +140,12 @@ fu_steelseries_gamepad_write_firmware_chunk(FuSteelseriesGamepad *self,
 								    error))
 		return FALSE;
 	chunk_checksum =
-	    fu_sum16(st_req->data + FU_STRUCT_STEELSERIES_GAMEPAD_WRITE_CHUNK_REQ_OFFSET_DATA,
+	    fu_sum16(st_req->buf->data + FU_STRUCT_STEELSERIES_GAMEPAD_WRITE_CHUNK_REQ_OFFSET_DATA,
 		     FU_STRUCT_STEELSERIES_GAMEPAD_WRITE_CHUNK_REQ_SIZE_DATA);
 	fu_struct_steelseries_gamepad_write_chunk_req_set_checksum(st_req, chunk_checksum);
 	*checksum += (guint32)chunk_checksum;
 
-	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req, error)) {
+	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req->buf, error)) {
 		g_prefix_error(error, "unable to flash block %u: ", fu_chunk_get_idx(chunk));
 		return FALSE;
 	}
@@ -188,8 +188,8 @@ fu_steelseries_gamepad_write_checksum(FuSteelseriesGamepad *self, guint32 checks
 	g_autoptr(GByteArray) buf_res = NULL;
 
 	fu_struct_steelseries_gamepad_write_checksum_req_set_checksum(st_req, checksum);
-	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req, error)) {
-		g_prefix_error(error, "unable to write checksum: ");
+	if (!fu_steelseries_device_request(FU_STEELSERIES_DEVICE(self), st_req->buf, error)) {
+		g_prefix_error_literal(error, "unable to write checksum: ");
 		return FALSE;
 	}
 	buf_res = fu_steelseries_device_response(FU_STEELSERIES_DEVICE(self), error);
@@ -202,7 +202,7 @@ fu_steelseries_gamepad_write_checksum(FuSteelseriesGamepad *self, guint32 checks
 									0x0,
 									error);
 	if (st_res == NULL) {
-		g_prefix_error(error, "controller is unable to validate checksum: ");
+		g_prefix_error_literal(error, "controller is unable to validate checksum: ");
 		return FALSE;
 	}
 
@@ -263,9 +263,10 @@ fu_steelseries_gamepad_write_firmware(FuDevice *device,
 }
 
 static void
-fu_steelseries_gamepad_set_progress(FuDevice *self, FuProgress *progress)
+fu_steelseries_gamepad_set_progress(FuDevice *device, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 1, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 93, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 1, "attach");
@@ -293,8 +294,6 @@ fu_steelseries_gamepad_class_init(FuSteelseriesGamepadClass *klass)
 static void
 fu_steelseries_gamepad_init(FuSteelseriesGamepad *self)
 {
-	fu_steelseries_device_set_iface_idx_offset(FU_STEELSERIES_DEVICE(self), -1);
-
 	fu_device_set_remove_delay(FU_DEVICE(self), FU_DEVICE_REMOVE_DELAY_RE_ENUMERATE);
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_BCD);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UNSIGNED_PAYLOAD);
