@@ -321,10 +321,11 @@ main(int argc, char *argv[])
 			FU_EFIVARS_GUID_SECURITY_DATABASE,
 			"dbx",
 			blob,
-			FU_EFIVARS_ATTR_APPEND_WRITE |
-			    FU_EFIVARS_ATTR_TIME_BASED_AUTHENTICATED_WRITE_ACCESS |
-			    FU_EFIVARS_ATTR_RUNTIME_ACCESS | FU_EFIVARS_ATTR_BOOTSERVICE_ACCESS |
-			    FU_EFIVARS_ATTR_NON_VOLATILE,
+			FU_EFI_VARIABLE_ATTR_APPEND_WRITE |
+			    FU_EFI_VARIABLE_ATTR_TIME_BASED_AUTHENTICATED_WRITE_ACCESS |
+			    FU_EFI_VARIABLE_ATTR_RUNTIME_ACCESS |
+			    FU_EFI_VARIABLE_ATTR_BOOTSERVICE_ACCESS |
+			    FU_EFI_VARIABLE_ATTR_NON_VOLATILE,
 			&error)) {
 			/* TRANSLATORS: dbx file failed to be applied as an update */
 			g_printerr("%s: %s\n", _("Failed to apply update"), error->message);
