@@ -21,7 +21,6 @@
 typedef struct {
 	FuEngine *engine;
 	GMainLoop *loop;
-	FuDaemonMachineKind machine_kind;
 	guint housekeeping_id;
 	gboolean update_in_progress;
 	gboolean pending_stop;
@@ -41,22 +40,6 @@ fu_daemon_get_engine(FuDaemon *self)
 	return priv->engine;
 }
 
-void
-fu_daemon_set_machine_kind(FuDaemon *self, FuDaemonMachineKind machine_kind)
-{
-	FuDaemonPrivate *priv = GET_PRIVATE(self);
-	g_return_if_fail(FU_IS_DAEMON(self));
-	priv->machine_kind = machine_kind;
-}
-
-FuDaemonMachineKind
-fu_daemon_get_machine_kind(FuDaemon *self)
-{
-	FuDaemonPrivate *priv = GET_PRIVATE(self);
-	g_return_val_if_fail(FU_IS_DAEMON(self), 0);
-	return priv->machine_kind;
-}
-
 void
 fu_daemon_set_update_in_progress(FuDaemon *self, gboolean update_in_progress)
 {
@@ -166,9 +149,7 @@ gboolean
 fu_daemon_setup(FuDaemon *self, const gchar *socket_address, GError **error)
 {
 	FuDaemonClass *klass = FU_DAEMON_GET_CLASS(self);
-	FuDaemonPrivate *priv = GET_PRIVATE(self);
 	FuEngine *engine = fu_daemon_get_engine(self);
-	const gchar *machine_kind = g_getenv("FWUPD_MACHINE_KIND");
 	guint timer_max_ms;
 	g_autoptr(FuProgress) progress = fu_progress_new(G_STRLOC);
 	g_autoptr(GTimer) timer = g_timer_new();
@@ -181,19 +162,6 @@ fu_daemon_setup(FuDaemon *self, const gchar *socket_address, GError **error)
 	if (!fu_daemon_check_syscall_filtering(error))
 		return FALSE;
 
-	/* allow overriding for development */
-	if (machine_kind != NULL) {
-		priv->machine_kind = fu_daemon_machine_kind_from_string(machine_kind);
-		if (priv->machine_kind == FU_DAEMON_MACHINE_KIND_UNKNOWN) {
-			g_set_error(error,
-				    FWUPD_ERROR,
-				    FWUPD_ERROR_INVALID_DATA,
-				    "invalid machine kind specified: %s",
-				    machine_kind);
-			return FALSE;
-		}
-	}
-
 	/* proxy on */
 	if (!klass->setup(self, socket_address, progress, error))
 		return FALSE;
@@ -231,6 +199,23 @@ fu_daemon_start(FuDaemon *self, GError **error)
 	g_return_val_if_fail(FU_IS_DAEMON(self), FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
+	/* super useful for debugging */
+	if (g_getenv("FWUPD_VERBOSE") != NULL) {
+		GHashTableIter iter;
+		const gchar *key;
+		const gchar *value;
+		g_autoptr(GHashTable) metadata = NULL;
+		g_autoptr(GString) str = g_string_new("report metadata:");
+
+		metadata = fu_engine_get_report_metadata(priv->engine, error);
+		if (metadata == NULL)
+			return FALSE;
+		g_hash_table_iter_init(&iter, metadata);
+		while (g_hash_table_iter_next(&iter, (gpointer *)&key, (gpointer *)&value))
+			g_string_append_printf(str, "\n - %s=%s", key, value);
+		g_debug("%s", str->str);
+	}
+
 	/* optional */
 	if (klass->start != NULL && !klass->start(self, error))
 		return FALSE;
